@@ -5,7 +5,7 @@ versionvfs.tcl --
 
 Written by Stephen Huntley (stephen.huntley@alum.mit.edu)
 License: Tcl license
-Version 1.03
+Version 1.5.2
 
 A versioning virtual filesystem.  Requires the template vfs in templatevfs.tcl.
 
@@ -19,7 +19,7 @@ Directories are versioned and tagged in the same way as files.
 Older versions can be retrieved by setting the -project and -time values appropriately.
 
 
-Usage: Mount ?-keep <number> -project <list of tags> -time <timestamp or "clock scan" suitable phrase>? <existing directory> <virtual directory>
+Usage: mount ?-keep <number> -project <list of tags> -time <timestamp or "clock scan" suitable phrase>? <existing directory> <virtual directory>
 
 Options:
 
@@ -72,12 +72,10 @@ The versioning vfs inherits the -cache and -volume options of the template vfs.
 ########################
 }
 
-package provide vfs::template::version 1.0
+package require vfs::template 1.5
+package require fileutil::globfind
 
-package require vfs::template
-
-package require globfind
-namespace import -force ::globfind::globfind
+package provide vfs::template::version 1.5.2
 
 namespace eval ::vfs::template::version {
 
@@ -224,12 +222,15 @@ proc file_delete {file} {
 proc file_executable {file} {
 	upvar path path root root relative relative
 	set fileName [VAcquireFile $path $root $relative]
+	if ![string first .&dir [file tail $fileName]] {return 0}
 	file executable $fileName
 }
 proc file_exists {file} {
 	upvar path path root root relative relative
 	set fileName [VAcquireFile $path $root $relative]
-	file exists $fileName
+	if ![string first .&dir [file tail $fileName]] {return 0}
+	if [file isdirectory $fileName] {return 1}
+	expr ![string equal [file join $path $relative] $fileName]
 }
 proc file_mkdir {file} {
 	upvar root root
@@ -245,6 +246,7 @@ proc file_mkdir {file} {
 proc file_readable {file} {
 	upvar path path root root relative relative
 	set fileName [VAcquireFile $path $root $relative]
+	if ![string first .&dir [file tail $fileName]] {return 0}
 	file readable $fileName
 }
 proc file_stat {file array} {
@@ -261,6 +263,7 @@ proc file_stat {file array} {
 proc file_writable {file} {
 	upvar path path root root relative relative
 	set fileName [VAcquireFile $path $root $relative]
+	if ![string first .&dir [file tail $fileName]] {return 0}
 	file writable $fileName
 }
 proc glob_ {directory dir nocomplain tails types typeString dashes pattern} {
@@ -296,7 +299,7 @@ proc open_ {file mode} {
 	if {$fileName == [file join $path $relative]} {
 		set fileName [VFileNameEncode [file join $path $relative]]\;[VCreateTag $root]
 		close [open $fileName $mode]
-		set channelID [vfs::memchan]
+		set channelID [memchan]
 		set ::vfs::template::version::filestats($channelID) "filename [list $fileName] hash [list $hash]"
 		return $channelID
 	}
@@ -307,7 +310,7 @@ proc open_ {file mode} {
 	fconfigure $f -translation binary
 	set hash [Hash $f]
 	close $f
-	set filed [vfs::memchan]
+	set filed [memchan]
 	if {[string index $mode 0] == "a"} {
 		set f [open $fileName r]
 		fconfigure $f -translation binary
@@ -336,6 +339,8 @@ proc MountProcedure {args} {
 
 # add custom handling for new vfs args here.
 
+	namespace import -force ::fileutil::globfind::globfind
+
 	set argsLength [llength $args]
 	for {set i 0} {$i < $argsLength} {incr i} {
 		switch -- [lindex $args $i] {
@@ -398,7 +403,7 @@ proc Hash {channel} {
 	::md5::md5 -hex -- [read $channel]
 }
 
-# figure out if time is a string, milliseconds or seconds count, return seconds cound
+# figure out if time is a string, milliseconds or seconds count, return seconds count
 proc SetTime {time} {
 	if ![string is digit -strict $time] {catch {set time [clock scan $time]}}
 	if ![string is digit -strict $time] {error "invalid time value."}
